require 'optparse'

ARGV[1] ||= "--help"

options = {}
option_parser = OptionParser.new do |opts|
  opts.banner = "Usage: nyaa remove <anime>\n" <<
                "       nyaa rm ...\n" <<
                "       nyaa remove -n <index>\n" <<
                "       nyaa remove -a\n\n" <<
                "       If you like this tool, please donate via PayPal\n" <<
                "       to wujoshuawu@gmail.com to keep the developer afloat.\n\n"

  opts.on("-a", "--all", "Remove all files") do
    options[:remove_all] = true
  end

  opts.on("-n", "--number [N]", "Remove torrents of an anime distribution",
                                "as numbered by `nyaa list`") do |n|
    options[:number] = n.to_i
  end
end
begin
  option_parser.parse!
rescue OptionParser::InvalidOption => e
  warn e
  puts option_parser
  exit 1
end

require 'nyaa_anime'

ENTRIES_BY_DISTRO = NyaaAnime.downloaded_entries_by_distro
distros = ENTRIES_BY_DISTRO.keys.sort!

if distros.size == 0
  puts "No files in #{NyaaAnime::NYAA_DL_DIR}."
end

require 'highline/import'

if options[:remove_all]
  ts = NyaaAnime.downloaded_entries.map { |entry| "#{NyaaAnime::NYAA_DL_DIR}#{entry.title}" }
  if agree "Delete all #{ts.size} torrent files? (y/n) "
    File.delete(*ts)
  end
  exit
end

def delete_distro(distro)
  ts = ENTRIES_BY_DISTRO[distro].map { |entry| "#{NyaaAnime::NYAA_DL_DIR}#{entry.title}" }
  if agree "Delete all #{ts.size} torrent files for #{distro}? (y/n) "
    File.delete(*ts)
  end
end

if options[:number]
  num = options[:number]
  if (1..distros.size).include? num
    delete_distro distros[num-1]
  else
    warn "Error: Number (#{num}) out of range 1-#{distros.size}."
    exit 1
  end
  exit
end

term = ARGV[1..-1].join " "
distros = ENTRIES_BY_DISTRO.keys.select { |distro| !!distro[/#{term}/i] }
if distros.count == 0
  warn "Error: No torrent files matching \"#{term}\"."
  exit 1
end
distros.each { |distro| delete_distro distro }
