FANSUBBERS = { "c"=>"Commie",
               "ch"=>"Chihiro",
               "f"=>"FFF",
               "h"=>"Hatsuyuki",
               "hs"=>"HorribleSubs" }
SPEEDSUBBER = FANSUBBERS["hs"]
SMALL_RESOLUTION = "480"
MEDIUM_RESOLUTION = "720"
LARGE_RESOLUTION = "1080"

require 'optparse'

ARGV[1] ||= "--help"

options = {}
option_parser = OptionParser.new do |opts|
  opts.banner = "Usage: nyaa search [options] <anime>\n" <<
                "       nyaa s [options] <anime>\n" <<
                "       nyaa search -n [-a [-q]]\n" <<
                "       nyaa get (equivalent to `nyaa search -naq`)\n\n" <<
                "       If you like this tool, please donate via PayPal\n" <<
                "       to wujoshuawu@gmail.com to keep the developer afloat.\n\n"

  opts.on("-i", "--individual", "List all results individually") do
    options[:list_individually] = true
  end
  opts.on("-F", "--no-filter", "Do not filter") do
    options[:no_filter] = true
  end

  opts.on("-n", "--new", "Find all new episodes of",
          "previously downloaded anime") do
    options[:find_new] = true
  end

  opts.on("-a", "--all", "Download all without asking") do
    options[:download_all] = true
  end
  opts.on("-q", "--quiet", "Be quiet (only with -a)") do
    options[:quiet] = true
  end

  opts.on("-s", "--small-res", "Add '#{SMALL_RESOLUTION}' to the search term") do
    options[:resolution] = SMALL_RESOLUTION
  end
  opts.on("-m", "--medium-res", "Add '#{MEDIUM_RESOLUTION}' to the search term") do
    options[:resolution] = MEDIUM_RESOLUTION
  end
  opts.on("-l", "--large-res", "Add '#{LARGE_RESOLUTION}' to the search term") do
    options[:resolution] = LARGE_RESOLUTION
  end

  opts.on("-f", "--fansubber [FANSUBBER]",
          "Specify [FANSUBBER] to be added",
          "to the search term. [FANSUBBER]",
          "can be abbreviated by",
          *FANSUBBERS.map {|o, f| "#{o} - #{f}"}) do |f|
    if FANSUBBERS[f]
      options[:subber] = FANSUBBERS[f]
    elsif f.nil?
      warn "Error: Expected [FANSUBBER] with the -f option"
      ARGV.push "--help"
    else
      options[:subber] = f
    end
  end

  opts.on("-S", "--speedsubs", "Equivalent to",
          "`--fansubber=#{SPEEDSUBBER}`") do
    options[:subber] = SPEEDSUBBER
  end
end
begin
  option_parser.parse!
rescue OptionParser::InvalidOption => e
  warn e
  puts option_parser
  exit 1
end

if !options[:download_all]
  options[:quiet] = false
end

require 'nyaa_anime'
require 'colorize'

if options[:find_new]
  entries = NyaaAnime.find_new_titles(!!options[:quiet]).sort!
  if entries.count == 0
    puts "No new episodes found."
    exit
  else
    puts "#{entries.count} new episode#{"s" if entries.count > 1} found!".colorize(NyaaAnime::NEW_COLOR)
    entries.each_with_index { |entry, i| puts "#{i+1}: #{entry.to_s.colorize(NyaaAnime::NEW_COLOR)}" }
  end
else
  search_term = ARGV[1..-1].join " "
  search_term = "[#{options[:subber]}] #{search_term}" if options[:subber]
  search_term += " #{options[:resolution]}" if options[:resolution]
  if search_term == ""
    warn "Error: No search term provided."
    exit 1
  end
  puts "Searching for \"#{search_term}\"..." unless options[:quiet]
  entries = options[:no_filter] ? NyaaAnime.search(search_term, false) : NyaaAnime.search(search_term)
  colorized_entries = NyaaAnime.colorized_entries entries.sort!
  new_entries = colorized_entries[:new]
  entries = colorized_entries.values.flatten!
  if options[:list_individually]
    entries.each_with_index { |entry, i| puts "#{i+1}: #{entry}" } unless options[:quiet]
  else
    entries = NyaaAnime.entries_by_distro(entries).to_a
    unless options[:quiet]
      entries.each_with_index do |d_e, i|
        distro, ents = d_e
        if ents.count == 1
          puts "#{i+1}: #{ents.first}"
          next
        end
        puts "#{i+1}: #{"([#{ents.count}] #{ents.first.first_ep}~#{ents.last.last_ep})".bold} #{distro}"
      end
    end
  end
  if entries.count == 0
    puts "No results for \"#{search_term}\"."
    exit
  else
    puts "#{entries.count} result#{"s" if entries.count > 1}, #{new_entries.count} new."
  end
end

require 'os'
require 'highline/import'

def prompt_to_open(torrents)
  torrents = [torrents] unless torrents.is_a? Array
  if torrents.count == 1
    puts "Downloaded #{File.basename torrents[0]}."
  else
    puts "Downloaded #{torrents.count} torrent files."
  end
  if agree "Open downloaded torrent#{"s" if torrents.count > 1}? (y/n): "
    if OS.mac?
      `open #{torrents.join " "}`
    elsif OS.windows?
      torrents.each { |torr| `#{torr}` }
    else # POSIX assumed
      torrents.each { |torr| `xdg-open #{torr} &` }
    end
  end
end

if options[:download_all]
  prompt_to_open NyaaAnime.download entries
  exit
end

if entries.count > 1
  prompt = "Download (1-#{entries.count}, 'a(ll)'" <<
           "#{", 'n(ew)'" if new_entries.count > 0 && !options[:find_new]}" <<
           ", or [ENTER] to cancel): "
  choice = ask(prompt) do |q|
    q.validate = Proc.new do |input|
      input == "" ||
      ("new"[/\A#{input}/i] && new_entries.count > 0 && !options[:find_new]) ||
      "all"[/\A#{input}/i] ||
      (1..entries.count).include?(input.to_i)
    end
    q.responses[:not_valid] = ""
  end

  if choice == ""
  elsif choice[/\An/i]
    prompt_to_open NyaaAnime.download new_entries
  elsif choice[/\Aa/i]
    prompt_to_open NyaaAnime.download entries
  else
    prompt_to_open NyaaAnime.download(entries.first.is_a?(Array) ? entries[choice.to_i-1][1] : entries[choice.to_i-1])
  end

else
  if agree "Download? (y/n): "
    prompt_to_open NyaaAnime.download(entries.first.is_a?(Array) ? entries.first[1].first : entries.first)
  end
end
